//
//  AppDelegate.m
//  Sample
//

#import "AppDelegate.h"
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>
#include <IOKit/usb/USBSpec.h>
#include <CommonCrypto/CommonDigest.h>

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    NSString *version = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
    NSString *framework = [IDT_BTPay SDK_version ];

    _window.title = [NSString stringWithFormat:@"BTPay Demo %@, Framework %@",version,framework];
}

-(id) returnClass{
    return [IDT_BTPay class];
}

-(void) displayUpRet2:(NSString*) operation returnValue: (RETURN_CODE)rt
{
    
    
    NSString * str = [NSString stringWithFormat:
                      @"\n%@ ERROR: ID-\"%i\", message: %@.\n",
                      operation, rt, [[IDT_BTPay sharedController] device_getResponseCodeString:rt]];
    [tv.textContainer.textView insertText:str];
    
    
}

- (IBAction) f_UploadJpeg: (id)sender{
    RETURN_CODE rt = [[IDT_BTPay sharedController] device_uploadJPEG:[NSData dataWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"IDTechLogo" ofType:@"jpg"]]];
    

    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Upload JPEG: OK."]];
    }
    else{
        [self displayUpRet2: @"Upload JPEG" returnValue: rt];
    }
    
    
    
}
- (IBAction) f_ShowJpeg: (id)sender{

    RETURN_CODE rt = [[IDT_BTPay sharedController] lcd_showJPEG:0 Y0:20 X1:127 Y1:43];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Show JPEG: OK."]];
    }
    else{
        [self displayUpRet2: @"Show JPEG" returnValue: rt];
        return;
    }
    
}

-(IBAction)open:(id)sender{
    [IDT_BTPay connectUSB:false];
    [[IDT_BTPay sharedController] setDelegate:self];
    [[IDT_BTPay sharedController] attemptConnect];

}


-(IBAction)openUSBBTPay:(id)sender{
    [IDT_BTPay connectUSB:true];
    [[IDT_BTPay sharedController] setDelegate:self];
    [[IDT_BTPay sharedController] attemptConnect];


}


-(IBAction)close:(id)sender{
    [[IDT_BTPay sharedController] close];

}
-(IBAction) getTerminalFile:(id)sender{

    TerminalData *t;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveTerminalData:&t];
    NSData *term = [NSData dataWithBytes:&t[0] length:44];
    
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Term Data: %@",term.description]];
    }
    else{
        [self displayUpRet2: @"Term Data" returnValue: rt];
        
    }
    
   
    
}
-(IBAction) removeTerminalFile:(id)sender{
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_removeTerminalData];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Remove Terminal File: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"Remove Terminal File" returnValue: rt];
        
    }
    
}

-(IBAction)createTerminalFile:(id)sender{


    //btpay
    TerminalData t;
    t.terminalCountryCode[0] = 0x08;
    t.terminalCountryCode[1] = 0x40;
    t.provideCardholderConfirmation = 0x01;
    t.terminalType = 0x22;  //21 online only, 22 offline with online, 23 ofline only
    t.emvContact = 0x01;
    t.terminalCapabilities[0] = 0xE0; //8=Manual Key Entry, 7=Magnetic Strip, 6=IC with contacts
    t.terminalCapabilities[1] = 0x20;  //8=Plainext Pin, 7=Enciphered Pin for Online, 6 = Signature, 5 = Enciphered Pin Offline, 4= No CVM
    t.terminalCapabilities[2] = 0xE8;  //8=SDA, 7=DDA, 6=Card Capture, 5=RFU, 4=CDA
    t.additionalTerminalCapabilities[0] = 0x40;  //8=cash, 7=goods, 6=services, 5=cashback, 4=inquiry, 3=transfer, 2=payment, 1=admin
    t.additionalTerminalCapabilities[1] = 0x00;  //8=cash deposit
    t.additionalTerminalCapabilities[2] = 0x80;  //8=numeric keys, 7=alpha-special keys, 6=command keys, 5=function keys
    t.additionalTerminalCapabilities[3] = 0x10;  //8=Print attend, 7=Print, cardholder, 6=display atten, 5=display cardholder, 4/3=RUF, 2=code10, 1=code9
    t.additionalTerminalCapabilities[4] = 0x01;  //8=code8.... 1=code1
    t.emvContactless = 0x01;
    t.magstripe = 0x01;
    t.pinTimeOut = 0x20;
    t.batchManaged = 0x00;
    t.adviceManaged = 0x00;
    t.pse = 0x00;
    t.autoRun = 0x00;
    t.predefinedAmount[0]= 0x00;
    t.predefinedAmount[1]= 0x00;
    t.predefinedAmount[2]= 0x00;
    t.pinByPass = 0x00;
    t.referalManaged = 0x01;
    t.defaultTAC = 0x01;
    t.defaultTACDefault[0] = 0x00;
    t.defaultTACDefault[1] = 0x00;
    t.defaultTACDefault[2] = 0x80;
    t.defaultTACDefault[3] = 0x00;
    t.defaultTACDefault[4] = 0x00;
    t.defaultTACDenial[0] = 0x00;
    t.defaultTACDenial[1] = 0x00;
    t.defaultTACDenial[2] = 0x00;
    t.defaultTACDenial[3] = 0x00;
    t.defaultTACDenial[4] = 0x00;
    t.defaultTACOnline[0] = 0x00;
    t.defaultTACOnline[1] = 0x00;
    t.defaultTACOnline[2] = 0x00;
    t.defaultTACOnline[3] = 0x00;
    t.defaultTACOnline[4] = 0x00;
    t.notRTS = 0x00;
    t.notVelocity = 0x00;
    t.cdaType = 0x02;
   
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_setTerminalData:t];
    
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Create Terminal File: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"Create Terminal File" returnValue: rt];
        
    }
}

-(IBAction) createAIDFile:(id)sender{

    ApplicationID a;
    
    for (int x = 0; x < 252; x++) {
        a.tdolValue[x] = 0x00;
        a.ddolValue[x] = 0x00;
        if (x < 5) {
            a.tACDefault[x] = 0x00;
            a.tACDenial[x] = 0x00;
            a.tACOnline[x] = 0x00;
        }

    }
    

    
    a.acquirerIdentifier[0] = 0x00;
    a.acquirerIdentifier[1] = 0x00;
    a.acquirerIdentifier[2] = 0x00;
    a.acquirerIdentifier[3] = 0x00;
    a.acquirerIdentifier[4] = 0x00;
    a.acquirerIdentifier[5] = 0x00;
    a.aid[0] = 0xa0;
    a.aid[1] = 0x00;
    a.aid[2] = 0x00;
    a.aid[3] = 0x00;
    a.aid[4] = 0x25;
    a.aid[5] = 0x01;
    a.aid[6] = 0x08;
    a.aid[7] = 0x02;
    a.aid[8] = 0x00;
    a.aid[9] = 0x00;
    a.aid[10] = 0x00;
    a.aid[11] = 0x00;
    a.aid[12] = 0x00;
    a.aid[13] = 0x00;
    a.aid[14] = 0x00;
    a.aid[15] = 0x00;
    a.aidLen = 0x08;
    a.applicationSelectionIndicator = 0x01;
    a.applicationVersionNumber[0] = 0x00;
    a.applicationVersionNumber[1] = 0x01;
    a.XAmount[0]=0x00;
    a.XAmount[1]= 0x00;
    a.XAmount[2] = 0x00;
    a.YAmount[0] = 0x00;
    a.YAmount[1] = 0x00;
    a.YAmount[2] = 0x00;
    a.skipTACIACDefault=0x01;
    a.tac = 0x01;
    a.floorlLimitChecking = 0x01;
    a.randomTransactionSelection = 0x01;
    a.velocitiyChecking=0x01;
    //tacdenial[5]
    //taconline[5]
    //tacdefault[5]
    a.terminalFloorLimit[0] = 0x00;
    a.terminalFloorLimit[1] = 0x3A;
    a.terminalFloorLimit[2] = 0x98;
    a.targetPercentage = 0x32;
    a.thresholdValue[0] = 0x00;
    a.thresholdValue[1] = 0x27;
    a.thresholdValue[2] = 0x10;
    a.maxTargetPercentage = 0x32;
    a.defaultTDOL = 0x01;
    a.tdolValue[0] = 0x9F;
    a.tdolValue[1] = 0x08;
    a.tdolValue[2] = 0x02;
    a.tdolLen = 0x03;
    a.defaultDDOL=0x01;
    a.ddolValue[0] = 0x9F;
    a.ddolValue[1] = 0x02;
    a.ddolValue[2] = 0x06;
    a.ddolLen = 0x03;
    a.transactionCurrencyCode[0]=0x08;
    a.transactionCurrencyCode[1]=0x40;
    a.transactionCurrencyExponent = 0x02;
    
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_setApplicationData:a];
    
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Create AID File: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"Create AID File" returnValue: rt];
        
    }
}
-(IBAction) getAIDFile:(id)sender{

    ApplicationID *t;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveApplicationData:@"a000000025010802" response:&t];
    
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        NSData *data = [NSData dataWithBytes:&t[0] length:sizeof(t)];
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Get AID File: %@",data.description]];
    }
    else{
        [self displayUpRet2: @"Get AID File" returnValue: rt];
        
    }
   
}
-(IBAction) removeAIDFile:(id)sender{

    [[IDT_BTPay sharedController] emv_removeApplicationData:@"a000000025010802"];
    
}
-(IBAction) getAIDFList:(id)sender{

    NSArray *dict;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveAIDList:&dict];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Get AID File List: %@",dict.description]];
    }
    else{
        [self displayUpRet2: @"Get AID File List" returnValue: rt];
        
    }
}

-(IBAction) createCAPKFile:(id)sender{

    CAKey a;
    a.rid[0] = 0xa0;
    a.rid[1] = 0x00;
    a.rid[2] = 0x00;
    a.rid[3] = 0x00;
    a.rid[4] = 0x03;
    a.index = 0x01;
    a.exponentLength = 0x01;
    
    a.exponent[0] = 0x03;
    a.exponent[1] = 0x00;
    a.exponent[2] = 0x00;
    NSData*d = [self hexToData:@"C696034213D7D8546984579D1D0F0EA519CFF8DEFFC429354CF3A871A6F7183F1228DA5C7470C055387100CB935A712C4E2864DF5D64BA93FE7E63E71F25B1E5F5298575EBE1C63AA617706917911DC2A75AC28B251C7EF40F2365912490B939BCA2124A30A28F54402C34AECA331AB67E1E79B285DD5771B5D9FF79EA630B75"];
    a.keyLength = d.length;
    Byte *keyByte = (Byte *)d.bytes;
    for (int x = 0; x < 248; x++) {
        if (x < a.keyLength) {
            a.key[x] = keyByte[x];
        }
        else{
            a.key[x] = 0;
        }
    }

    int rt = [[IDT_BTPay sharedController] emv_setCAPK:a];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Create CAPK: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"Create CAPK" returnValue: rt];
    }
    
    
    
}
-(IBAction) getCAPKFile:(id)sender{

    CAKey* key;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveCAPK:@"a000000003" index:@"01" response:&key];
    
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        NSData *data = [NSData dataWithBytes:&key[0] length:sizeof(key)];
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"getCAPKFile File: %@",data.description]];
    }
    else{
        [self displayUpRet2: @"getCAPKFile File" returnValue: rt];
        return;
        
    }
    if (key == nil) {
        [tv.textContainer.textView insertText:@"CAPK File Not Returned - Try Again"];
        return;
    }

    
}
-(IBAction) removeCAPKFile:(id)sender{

    int rt = [[IDT_BTPay sharedController] emv_removeCAPK:@"a000000003" index:@"01"];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"remove CAPK File: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"remove CAPK File" returnValue: rt];
        return;
        
    }
}
-(IBAction) getCAPKList:(id)sender{

    NSArray* dict;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveCAPKList:&dict];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"getCAPKList: %@",dict.description]];
    }
    else{
        [self displayUpRet2: @"getCAPKList" returnValue: rt];
        return;
        
    }
}

-(IBAction) createCRLFile:(id)sender{

    CRLEntry c;
    c.rid[0] = 0xa0;
    c.rid[1] = 0x00;
    c.rid[2] = 0x00;
    c.rid[3] = 0x00;
    c.rid[4] = 0x03;
    c.index = 0x50;
    c.serialNumber[0] = 0x00;
    c.serialNumber[1] = 0x10;
    c.serialNumber[2] = 0x00;
    int rt = [[IDT_BTPay sharedController] emv_setCRL:c];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"create CRL file: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"create CRL file" returnValue: rt];
        return;
        
    }
}

-(IBAction) iccStatus:(id)sender{
    ICCReaderStatus* dict;
   RETURN_CODE rt = [[IDT_BTPay sharedController] icc_getICCReaderStatus:&dict];
    NSData *data = [NSData dataWithBytes:&dict[0] length:sizeof(dict)];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"iccStatus File: %@",data.description]];
    }
    else{
        [self displayUpRet2: @"iccStatus File" returnValue: rt];
        return;
        
    }


    NSLog(@"icc power = %@",dict->iccPower?@"True":@"False");
    NSLog(@"cardSeated = %@",dict->cardSeated?@"True":@"False");
    NSLog(@"latchClosed = %@",dict->latchClosed?@"True":@"False");
    NSLog(@"cardPresent = %@",dict->cardPresent?@"True":@"False");
    NSLog(@"magneticDataPresent = %@",dict->magneticDataPresent?@"True":@"False");
}
-(IBAction) getCRLFile:(id)sender{

    NSArray* returnArray;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveCRLForRID:@"a000000003" response:&returnArray];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"getCRLFile: %@",returnArray.description]];
    }
    else{
        [self displayUpRet2: @"getCRLFile" returnValue: rt];
        return;
        
    }
    NSLog(@"ar = %@",returnArray.description);

}
-(IBAction) removeCRLFile:(id)sender{

    CRLEntry c;
    c.rid[0] = 0xa0;
    c.rid[1] = 0x00;
    c.rid[2] = 0x00;
    c.rid[3] = 0x00;
    c.rid[4] = 0x03;
    c.index = 0x50;
    c.serialNumber[0] = 0x00;
    c.serialNumber[1] = 0x10;
    c.serialNumber[2] = 0x00;
    int rt = [[IDT_BTPay sharedController] emv_removeCRLUnit:c];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"removeCRLFile: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"removeCRLFile" returnValue: rt];
        return;
    }
}
-(IBAction) getCRLList:(id)sender{

    NSArray* dict;
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_retrieveCRLList:&dict];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"getCRLList: %@",dict.description]];
    }
    else{
        [self displayUpRet2: @"getCRLList" returnValue: rt];
        return;
    }
    NSLog(@"array = %@",dict.description);
}


- (void) emvTransactionData:(IDTEMVData*)emvData errorCode:(int)error{
    
    [tv.textContainer.textView insertText: [NSString stringWithFormat:@"unencrypted = %@",emvData.unencryptedTags]];
    [tv.textContainer.textView insertText: [NSString stringWithFormat:@"encrypted = %@",emvData.encryptedTags]];
    [tv.textContainer.textView insertText: [NSString stringWithFormat:@"masked = %@",emvData.maskedTags]];
    [tv.textContainer.textView insertText: [NSString stringWithFormat:@"ksn = %@",emvData.KSN]];
    [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Result code = %i, encryption mode %i, cardtype %i",emvData.resultCode,emvData.encryptionMode, emvData.cardType]];

    
    NSLog(@"unencrypted = %@",emvData.unencryptedTags);
    NSLog(@"encrypted = %@",emvData.encryptedTags);
    NSLog(@"masked = %@",emvData.maskedTags);
    NSLog(@"ksn = %@",emvData.KSN);
    NSLog(@"Result code = %i, encryption mode %i, cardtype %i",emvData.resultCode,emvData.encryptionMode, emvData.cardType);
}

-(void) clearMemory:(unsigned char*)loc length:(int)len{
    if (len < 1) {
        return;
    }
    for (int x = 0; x < len; x ++) {
        loc[x] = 0x00;
    }
}

-(IBAction)completeEMV:(id)sender{
    


    NSMutableDictionary* dict = [NSMutableDictionary new];

    [dict setObject:[self hexToData:@"3030"] forKey:@"8a"];
    [dict setObject:[self hexToData:@"11223344556677883030"] forKey:@"91"];
    [dict setObject:[self hexToData:@"9f20041122334486098424000004aabbccdd"] forKey:@"71"];
    [dict setObject:[self hexToData:@"9f20041122334486098424000004aabbccdd"] forKey:@"72"];
    IDTEMVData* emvData;
    int rt = [[IDT_BTPay sharedController] emv_completeOnlineEMVTransaction:true hostResponseTags:dict responseTags:&emvData];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"unencrypted = %@",emvData.unencryptedTags]];
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"encrypted = %@",emvData.encryptedTags]];
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"masked = %@",emvData.maskedTags]];
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"ksn = %@",emvData.KSN]];
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Result code = %i, encryption mode %i, cardtype %i",emvData.resultCode,emvData.encryptionMode, emvData.cardType]];
    }
    else{
        [self displayUpRet2: @"completeEMV" returnValue: rt];
        return;
    }
    
    NSLog(@"Response Code = %i",rt);
    NSLog(@"unencrypted = %@",emvData.unencryptedTags);
    NSLog(@"encrypted = %@",emvData.encryptedTags);
    NSLog(@"masked = %@",emvData.maskedTags);
    NSLog(@"ksn = %@",emvData.KSN);
    NSLog(@"Result code = %i, encryption mode %i, cardtype %i",emvData.resultCode,emvData.encryptionMode, emvData.cardType);

    
    NSLog(@"Response Code = %i",rt);
    NSLog(@"unencrypted = %@",emvData.unencryptedTags);
    NSLog(@"encrypted = %@",emvData.encryptedTags);
    NSLog(@"masked = %@",emvData.maskedTags);
    NSLog(@"ksn = %@",emvData.KSN);
    NSLog(@"Result code = %i, encryption mode %i, cardtype %i",emvData.resultCode,emvData.encryptionMode, emvData.cardType);
    
}



-(NSData*) computeHash:(NSData*) data{
    
    CC_LONG bufferlen = (CC_LONG)data.length;
    Byte* buffer = malloc(bufferlen);
    [data getBytes:buffer];
    unsigned char digest[20];
    CC_SHA1(buffer, bufferlen, digest);
    free(buffer);
    NSData* retValue = [NSData dataWithBytes:digest length:20];
    return retValue;
}


-(IBAction)startEMV:(id)sender{

    [emvLabel setStringValue:@"TRANSACTION STARTING"];
    NSMutableDictionary* tags;
    //[[IDT_BTPay sharedController] sendDataCommand:[self hexToData:@"724601200080009c01099f02060000000025009f0306000000000500"] calcLRC:YES response:&d];
    RETURN_CODE rt = [[IDT_BTPay sharedController] emv_startEMVTransaction:2.95 otherAmount:0.00 timeout:30 transactionType:0 additionalTags:tags];
    
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"startEMV: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"startEMV" returnValue: rt];
        return;
    }
    
}

-(void) deviceConnected{
    [connectedLabel setStringValue:@"DEVICE CONNECTED"];
}
-(void) deviceDisconnected{
    [connectedLabel setStringValue:@"DEVICE DISCONNECTED"];
}

- (void) dataInOutMonitor:(NSData*)data  incoming:(BOOL)isIncoming{
    NSLog(@"Received Data====================================");
    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\n%@: %@",isIncoming?@"IN":@"OUT",data.description]];
}




- (void) pinpadData:(NSData*)value keySN:(NSData*)KSN event:(EVENT_PINPAD_Types)event{
    NSLog(@"--PIN event Received, Type: %d, data: %@,  ksn: %@", event, value.description,KSN.description);
    switch (event) {
        case EVENT_PINPAD_UNKNOWN:
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) PINPad unknown event, data: %@", value.description]];
            break;
            
        case EVENT_PINPAD_ENCRYPTED_PIN:
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) Encrypted PIN data  received: %@", value.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event)  KSN data  received: %@", KSN.description]];
            break;
        case EVENT_PINPAD_FUNCTION_KEY:
            if ([value.description isEqualToString:@"<43>"]) [tv.textContainer.textView insertText:@"CANCEL KEY PRESSED"];
            if ([value.description isEqualToString:@"<42>"]) [tv.textContainer.textView insertText:@"BACKSPACE KEY PRESSED"];
            if ([value.description isEqualToString:@"<45>"]) [tv.textContainer.textView insertText:@"ENTER KEY PRESSED"];
            if ([value.description isEqualToString:@"<23>"]) [tv.textContainer.textView insertText:@"# KEY PRESSED"];
            if ([value.description isEqualToString:@"<2a>"]) [tv.textContainer.textView insertText:@"* KEY PRESSED"];
            
            break;
            
        case EVENT_PINPAD_NUMERIC:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) Numeric data  received: %@", [[NSString alloc] initWithData:value encoding:NSASCIIStringEncoding]]];
            /*
             aString = [[NSString alloc] initWithData: someData  encoding:NSUTF8StringEncoding];
             [self appendMessageToResults: [NSString stringWithFormat:@"Numeric is : %@", aString]];
             [aString release];
             */
        }
            break;
        case EVENT_PINPAD_AMOUNT:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) Amount data  received: %.2f", [[[NSString alloc] initWithData:value encoding:NSASCIIStringEncoding] floatValue]/100]];
            /*
             aString = [[NSString alloc] initWithData: someData encoding:NSUTF8StringEncoding];
             [self appendMessageToResults: [NSString stringWithFormat:@"Amount is : %@", aString]];
             [aString release];
             */
        }
            break;
            
        case EVENT_PINPAD_ACCOUNT:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) Encrypted Account data  received: %@", value.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) KSN data  received: %@", KSN.description]];
            /*
             aString = [[NSString alloc] initWithData: someData encoding:NSUTF8StringEncoding];
             [self appendMessageToResults: [NSString stringWithFormat:@"Account is : %@", aString]];
             [aString release];
             */
        }
            break;
            
        case EVENT_PINPAD_ENCRYPTED_DATA:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) Encrypted data  received: %@", value.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"(Event) Encrypted data  KSN: %@", KSN.description]];
        }
            break;
            
        case EVENT_PINPAD_CANCEL:
        {
            [tv.textContainer.textView insertText: @"(Event) PINPad Canceled."];
        }
            break;
            
        case EVENT_PINPAD_TIMEOUT:
        {
            [tv.textContainer.textView insertText:@"(Event) PINPad Timeout."];
        }
            break;
        case EVENT_PINPAD_DATA_ERROR:
        {
            [tv.textContainer.textView insertText:@"(Event) PINPad Data Error."];
        }
            break;
            
        default:
            break;
    }
}


- (void) swipeMSRData:(IDTMSRData*)cardData{
    switch (cardData.event) {
        case EVENT_MSR_CARD_DATA:
        {
            switch (cardData.captureEncodeType) {
                case CAPTURE_ENCODE_TYPE_ISOABA:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncode Type: %@", @"ISO/ABA"]];
                    break;
                case CAPTURE_ENCODE_TYPE_AAMVA:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncode Type: %@",  @"AA/MVA"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_Other:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncode Type: %@",  @"Other"]];
                    break;
                    
                case CAPTURE_ENCODE_TYPE_Raw:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncode Type: %@",  @"Raw"]];
                    break;
                    
                default:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncode Type: %@",  @"UNKNOWN"]];
                    
                    break;
            }
            switch (cardData.captureEncryptType) {
                case CAPTURE_ENCRYPT_TYPE_AES:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncryption Type: %@", @"AES"]];
                    break;
                case CAPTURE_ENCRYPT_TYPE_TDES:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncryption Type: %@",  @"TDES"]];
                    break;
                
                    
                default:
                    [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncryption Type: %@",  @"UNKNOWN"]];
                    
                    break;
            }
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nCard Data: %@",  cardData.cardData]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nTrack 1(%i): %@", cardData.track1Length, cardData.track1]];
            [tv.textContainer.textView insertText: [NSString stringWithFormat:@"\nTrack 2(%i): %@",  cardData.track2Length, cardData.track2]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nTrack 3(%i): %@",  cardData.track3Length, cardData.track3]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncoded Track 1: %@",  cardData.encTrack1.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncoded Track 2: %@",  cardData.encTrack2.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nEncoded Track 3: %@",  cardData.encTrack3.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nHash Track 1: %@",  cardData.hashTrack1.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nHash Track 2: %@",  cardData.hashTrack2.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nHash Track 3: %@",  cardData.hashTrack3.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nKSN: %@",  cardData.KSN.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nSessionID: %@",  cardData.sessionID.description]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nReader Serial Number: %@",  cardData.RSN]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nRead Status: %2X",  cardData.readStatus]];
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\nSession ID: %@",  cardData.sessionID.description]];

            return;
        }
            break;
            
        case EVENT_MSR_CANCEL_KEY:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\n(Event) MSR Cancel Key received: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_BACKSPACE_KEY:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\n(Event) MSR Backspace Key received: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_ENTER_KEY:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\n(Event) MSR Enter Key received: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        case EVENT_MSR_UNKNOWN:
        {
            [tv.textContainer.textView insertText:[NSString stringWithFormat:@"\n(Event) MSR unknown event, data: %@",  cardData.encTrack1]];
            return;
        }
            break;
            
        default:
            break;
    }
}

-(IBAction) sendAPDU:(id)sender{
    NSString* apduString = [apdu.cell title];
    APDUResponse  *retVal = nil;
    
    RETURN_CODE rt = [[IDT_BTPay sharedController] icc_exchangeAPDU:[self hexToData:apduString] response:&retVal];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"sendAPDU: %@",retVal.response.description]];
    }
    else{
        [self displayUpRet2: @"sendAPDU" returnValue: rt];
        return;
    }
}



-(IBAction) msrON:(id)sender{

    RETURN_CODE rt = [[IDT_BTPay sharedController] msr_startMSRSwipeWithDisplay:@"Please" line2:@"Swipe" line3:@"Card"];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"msrON: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"msrON" returnValue: rt];
        return;
    }
}
-(IBAction) msrOff:(id)sender{

    RETURN_CODE rt = [[IDT_BTPay sharedController] msr_cancelMSRSwipe];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"msrOFF: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"msrOFF" returnValue: rt];
        return;
    }


}
-(IBAction) powerOnICC:(id)sender{

    NSData *retVal = nil;
    RETURN_CODE rt = [[IDT_BTPay sharedController] icc_powerOnICC:&retVal ];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"powerOnICC: %@",retVal.description]];
    }
    else{
        [self displayUpRet2: @"powerOnICC" returnValue: rt];
        return;
    }
}

-(IBAction) powerOffICC:(id)sender{
    NSString *retVal = nil;
    RETURN_CODE rt = [[IDT_BTPay sharedController] icc_powerOffICC:&retVal ];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"powerOffICC: %@",retVal.description]];
    }
    else{
        [self displayUpRet2: @"powerOffICC" returnValue: rt];
        return;
    }

    
}
-(IBAction) requestPIN:(id)sender{
    RETURN_CODE rt = [[IDT_BTPay sharedController] pin_getEncryptedPIN:@"1234567890123456" keyType:PIN_KEY_TDES_DUKPT_extp line1:@"Please" line2:@"input PIN" line3:nil];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"requestPIN: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"requestPIN" returnValue: rt];
        return;
    }
    
}

-(IBAction) requestNumeric:(id)sender{
    RETURN_CODE rt = [[IDT_BTPay sharedController] pin_getNumeric:false minLength:1 maxLength:16 messageID:1 language:LANGUAGE_TYPE_ENGLISH];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"requestNumeric: %@",@"OK"]];
    }
    else{
        [self displayUpRet2: @"requestNumeric" returnValue: rt];
        return;
    }
}

- (IBAction) f_getFirm:(id)sender
{    NSString *result;
    RETURN_CODE rt = [[IDT_BTPay sharedController]  device_getFirmwareVersion:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Get Firmware: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Firmware" returnValue: rt];
        return;
    }
 
}
- (IBAction) f_getModelNumber:(id)sender
{
    NSString* result;
    RETURN_CODE rt = [[IDT_BTPay sharedController] config_getModelNumber:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Get Model No: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Model No" returnValue: rt];
        return;
    }

}

- (IBAction) f_getSerialNumber:(id)sender
{
    NSString* result;
    RETURN_CODE rt = [[IDT_BTPay sharedController] config_getSerialNumber:&result];
    if (RETURN_CODE_DO_SUCCESS == rt)
    {
        [tv.textContainer.textView insertText: [NSString stringWithFormat:@"Get Serial No: %@",result]];
    }
    else{
        [self displayUpRet2: @"Get Serial Number" returnValue: rt];
        return;
    }

}

-(unsigned int) char2hex:(char)c{
	
	switch (c) {
		case '0' ... '9': return c - '0';
		case 'a' ... 'f': return c - 'a' + 10;
		case 'A' ... 'F': return c - 'A' + 10;
		default: return -1;
	}
}

- (NSData *)hexToData:(NSString*)str {   //Example - Pass string that contains characters "30313233", and it will return a data object containing ascii characters "0123"
	if ([str length] == 0) {
		return nil;
	}
	
	unsigned stringIndex=0, resultIndex=0, max=(int)[str length];
	NSMutableData* result = [NSMutableData dataWithLength:(max + 1)/2];
	unsigned char* bytes = [result mutableBytes];
	
	unsigned num_nibbles = 0;
	unsigned char byte_value = 0;
	
	for (stringIndex = 0; stringIndex < max; stringIndex++) {
		unsigned int val = [self char2hex:[str characterAtIndex:stringIndex]];
		
		num_nibbles++;
		byte_value = byte_value * 16 + (unsigned char)val;
		if (! (num_nibbles % 2)) {
			bytes[resultIndex++] = byte_value;
			byte_value = 0;
		}
	}
	
	
	//final nibble
	if (num_nibbles % 2) {
		bytes[resultIndex++] = byte_value;
	}
	
	[result setLength:resultIndex];
	
	return result;
}


@end
